//  This file is part of the Mussa source distribution.
//  http://mussa.caltech.edu/
//  Contact author: Tristan  De Buysscher, tristan@caltech.edu

// This program and all associated source code files are Copyright (C) 2005
// the California Institute of Technology, Pasadena, CA, 91125 USA.  It is
// under the GNU Public License; please see the included LICENSE.txt
// file for more information, or contact Tristan directly.


PathList
ExtendPathWindows(PathList raw_list)
{
  // to keep track of everything, this function acts like a finite state
  // automata:  3 bitfields: reset, extending, scanning - ie 8 states
  int reset = 0, extending = 0, scanning = 0;

  int species_num = raw_list.SpeciesNum();
  int win_num = raw_list.WinNum();

  int *cur_path, *next_path;

  cur_path = new int[species_num];
  next_path = new int[species_num];

  raw_list.ResetIndex();

  while (no more windows avaible)
  {
    //cur_indices[0] + ref_ext == next_indices[0] + 1:

    // state 000 - start/nominal search
    // 
    if !reset && !extending && !scanning
    {
      if cur_path[0] == next_path[0]
      {
        scanning = 1;                   // go to state 001
        raw_list.IncreIndex();
      }
      else if cur_path[0] + 1 == next_path[0]
      {
        extending = 1;                  // go to state 010
        win_ext = 1;
        raw_list.SetDirty();
        raw_list.IncreIndex();
      }
      else
        reset = 1;                      // go to state 100
    }

    // state 001 - scanning for ext window thru possible interleaving
    // 
    else if !reset && !extending && scanning
    {
      if cur_path[0] == next_path[0]
        raw_list.IncreIndex();
      else if cur_path[0] + 1 == next_path[0]
      {
        extending = 1;                  // go to state 011
        win_ext = 1;
        raw_list.SetDirty();
        raw_list.IncreIndex();
      }
      else
        reset = 1;                      // go to state 101
    }

    // state 010 - extending window with no interleaving
    // 
    else if !reset && extending && !scanning
    {
      if (cur_path[0] + win_ext) == next_path[0]
      {
        extending = 1;                  // go to state 011
        raw_list.IncreIndex();
      }
      else if (cur_path[0] + win_ext + 1) == next_path[0]
      {
        win_ext++;
        raw_list.SetDirty();
        raw_list.IncreIndex();
      }
      else
        reset = 1;                      // go to state 110
    }

    // state 011 - extending window with interleaving
    // 
    else if !reset && extending && scanning
    {
      if (cur_path[0] + win_ext) == next_path[0]
        raw_list.IncreIndex();
      else if (cur_path[0] + win_ext + 1) == next_path[0]
      {
        win_ext++;
        raw_list.SetDirty();
        raw_list.IncreIndex();
      }
      else
        reset = 1;                      // go to state 111
    }


    // reset condition states, return to 000 after taking appropiate action

    // state 100 - no extended window, no scanning happened
    // save basic window to extended window list
    if reset && !extending && !scanning
    {
      // return to start/nominal search state
      reset = 0;
    }

    // state 101 - no extended window, scanned for interleaving
    // save basic window to extended window list, return to first clean 
    else if reset && !extending && scanning
    {
      // return to start/nominal search state
      reset = 0;
      scanning = 0;
    }

    // state 110 - extended window, no interleaving
    // calc extended window, save to extended window list
    else if reset && extending && !scanning
    {
      // return to start/nominal search state
      reset = 0;
      extending = 0;
    }
       
    // state 111 - extended window, interleaved
    // calc extended window, save to ext win list, return to first clean
    else if reset && extending && scanning
    {        
      // return to start/nominal search state
      reset = 0;
      extending = 0;
      scanning = 0;
     }

  return ext_win;
}
