#include <QFrame>
#include <QList>
#include <QScrollArea>
#include <QVBoxLayout>

#include "qui/mussa_setup_dialog/SequenceSetupWidget.hpp"
#include "qui/mussa_setup_dialog/SequenceSetupFrame.hpp"
#include "qui/mussa_setup_dialog/SetupInfo.hpp"


SequenceSetupFrame::SequenceSetupFrame(QWidget *parent)
  : QWidget(parent),
    sequenceFrame(0),
    scrollArea(0),
    seqCount(1),
    default_directory(QDir::current()),
    sequenceLayout(0)
{

  // frame
  sequenceFrame = new QFrame;
  sequenceFrame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  
  sequenceLayout = new QVBoxLayout;  

  sequenceFrame->setLayout(sequenceLayout);
  sequenceFrame->resize(500,500);

  //scroll area
  scrollArea = new QScrollArea;
  scrollArea->setWidget(sequenceFrame);

  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(scrollArea);
  setLayout(mainLayout);

  //Initialize to one sequence
  changeSequenceCount(1);
  
  scrollArea->show();
}

QDir SequenceSetupFrame::get_default_dir() const 
{
  return default_directory;
}

void SequenceSetupFrame::set_default_dir(const QDir& d)
{
  default_directory = d;
}

void SequenceSetupFrame::changeSequenceCount(int newSeqCount)
{
  seqCount = newSeqCount;
  setupSequences(seqCount);
  resizeInnerFrame(seqCount);
}

void SequenceSetupFrame::resizeInnerFrame(int seqCount)
{
  if (seqCount >= 1)
  {
    sequenceFrame->resize(565, 125 * seqCount + 30);
  }
  else
  {
    sequenceFrame->resize(565, 125 + 30);
  }
}

void SequenceSetupFrame::setupSequences(int newSeqCount)
{
  const int curSeqCount = seqList.size();

  if (curSeqCount == newSeqCount)
  {
    return;
  }
  else if (curSeqCount < newSeqCount)
  {
    int numSeqsToAdd = newSeqCount - curSeqCount;

    SequenceSetupWidget *seq = 0;
    for (int i = 1; i <= numSeqsToAdd; i++)
    {
      seq = new SequenceSetupWidget(this);
      seqList.push_back(seq);
      sequenceLayout->addWidget(seq);
    } 
  }
  else if (curSeqCount > newSeqCount)
  {
    int numSeqsToRemove = curSeqCount - newSeqCount;
    
    if (numSeqsToRemove > curSeqCount)
    {
      numSeqsToRemove = curSeqCount;
    }
    
    SequenceSetupWidget *seqWidget = 0;
    for (int i=1; i <= numSeqsToRemove; i++)
    {
      seqWidget = seqList.back();
      sequenceLayout->removeWidget(seqWidget);
      seqList.pop_back();
      delete seqWidget;
    }

  }
  updateGeometry();  
}

std::list<SetupInfo *> SequenceSetupFrame::getSetupInfo()
{
  const int curSeqCount = seqList.size();
  
  std::list<SetupInfo *> setupInfoList;

  QString seqName;
  QString seqFile;
  QString annotFile;
  int fastaIndex;
  int start;
  int end;


  SequenceSetupWidget *seq = 0;
  //SetupInfo *setupInfo = 0;
  for (int i = 0; i < curSeqCount; i++)
  {
    seq = seqList.back();
   
    seqName = seq->getName();
    seqFile = seq->getSeqFile();
    annotFile = seq->getAnnotFile();
    fastaIndex = seq->getFastaIndex();
    start = seq->getStart();
    end = seq->getEnd();

    seqList.push_front(seq);
    seqList.pop_back();

    setupInfoList.push_back(new SetupInfo(seqName.toStdString(),
            seqFile.toStdString(), 
					  annotFile.toStdString(), 
					  fastaIndex, start, end));
    
  } 

  return setupInfoList;
}

